/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2011 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#include <unistd.h>
#include <fcntl.h>

#include <string.h>

char *client_config_array[][3]={
{"ip",			"192.168.1.202",	NULL},
{"port",		"3334",			NULL},
{"input_device",	NULL, 			NULL},
{"input_preset",	"touch", 		NULL},
};

char *client_config_value_get(char *arg)
{
	int i;
	int client_config_array_index=sizeof(client_config_array) / (sizeof(char *) * 3);

	for(i=0 ; i < client_config_array_index ; i++)
		if(strcmp(arg, client_config_array[i][0]) == 0)
			if(client_config_array[i][2] != NULL)
				return client_config_array[i][2];
			else
				return client_config_array[i][1];

	printf("Error: wrong config argument: '%s'.\n", arg);

	return NULL;
}

void client_config_value_set(char *arg, char *val)
{
	int i;
	int client_config_array_index=sizeof(client_config_array) / (sizeof(char *) * 3);

	for(i=0 ; i < client_config_array_index ; i++)
		if(strcmp(arg, client_config_array[i][0]) == 0)
		{
			client_config_array[i][2]=val;
			printf("Config: %s=%s\n", arg, val);
			return;
		}

	printf("Error: wrong config argument: '%s'.\n", arg);
	free(val);
}

void client_config_close()
{
	int i;
	int client_config_array_index=sizeof(client_config_array) / (sizeof(char *) * 3);

	for(i=0 ; i < client_config_array_index ; i++)
		free(client_config_array[i][2]);
}

void client_config_read(char *client_config_file)
{
	int client_config_fd=0;

	char c=0;
	char *arg=NULL;
	char *val=NULL;

	bool arg_lock=true;
	bool val_lock=true;
	bool comment_lock=false;

	int arg_count=0;
	int val_count=0;

	client_config_fd=open(client_config_file, O_RDONLY);
	if(client_config_fd < 0)
	{
		perror("Error while opening config file");
		return;
	}

	arg=malloc(256);
	memset(arg, 0, 256);

	val=malloc(256);
	memset(val, 0, 256);

	while(read(client_config_fd, &c, sizeof(char)))
	{
		if(c == '\n')
		{
			comment_lock=false;

			if(strcmp(arg, "") == 0 || strcmp(val, "") == 0)
				continue;

			client_config_value_set(arg, val);

			memset(arg, 0, 256);
			val=malloc(256);
			memset(val, 0, 256);

			arg_lock=true;
			val_lock=true;

			arg_count=0;
			val_count=0;

			continue;
		}
		if(c == '#')
		{
			comment_lock=true;
			continue;
		}
		if(comment_lock)
			continue;
		if(c == '=')
		{
			arg_lock=false;
			continue;
		}
		if((c == ' ' || c == '\t') && (arg_lock || val_lock) || c == '"' || c == '\'')
			continue;
	
		if(arg_lock)
		{
			arg[arg_count]=c;
			arg_count++;
		}
		else
		{
			if(val_lock)
				val_lock=false;

			val[val_count]=c;
			val_count++;
		}
	}

	close(client_config_fd);
}

