/*******************************************************************************
				Neoinput
				========

		Copyright (C) 2010 Paul Kocialkowski

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include <linux/input.h>
#include <linux/uinput.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

#include "server.h"
#include "input_read.h"
#include "input.h"

#include "input_preset.h"
#include "input_write.h"

int input=0;
char *input_device=NULL;
char *input_preset=NULL;

float speed_factor=1;

char *uinput_devices[3]={"/dev/input/uinput/", "/dev/uinput", "/dev/misc/uinput"};

void input_write_open(char *input_device)
{
	struct uinput_user_dev uinput_device;
	int i;

	/* Use uinput if nothing specified */
	if(input_device == NULL)
	{
		/* Try each uinput device */
		for(i=0 ; i < 3 ; i++)
		{
			input=open(uinput_devices[i], O_WRONLY | O_NONBLOCK);

			if(input < 0)
				continue;
			else
				break;
		}
	}
	else
		input=open(input_device, O_WRONLY);

	if(input < 0)
	{
		if(input_device == NULL)
			printf("Error while opening uinput.\n");
		else
			printf("Error while opening %s.\n", input_device);
		exit(1);
	}
	printf("input is %s\n", uinput_devices[i]);

	/*** BEGIN : Testing purpose only ***/
	/* Tu check les devices du preset pour voir quoi ioctl-er */
	/* En effet, y penser */

	/* Ajouter les EVBITS */
	ioctl(input, UI_SET_EVBIT, EV_KEY);
	ioctl(input, UI_SET_EVBIT, EV_REL); 
	ioctl(input, UI_SET_EVBIT, EV_SYN);

	if(input_preset_value_bool("use_touch"))
	{
		if(input_preset_value_comp("touch_action", "mouse") || input_preset_value_comp("touch_action", "mouse_rotated"))
		{
			ioctl(input, UI_SET_RELBIT, REL_X);
			ioctl(input, UI_SET_RELBIT, REL_Y);
		}
		if(input_preset_value_comp("touch_action", "arrows"))
		{
			if(input_preset_value_isset("touch_arrows_up_key"))
				ioctl(input, UI_SET_KEYBIT, atoi(input_preset_value_get("touch_arrows_up_key")));
			if(input_preset_value_isset("touch_arrows_down_key"))
				ioctl(input, UI_SET_KEYBIT, atoi(input_preset_value_get("touch_arrows_down_key")));
			if(input_preset_value_isset("touch_arrows_left_key"))
				ioctl(input, UI_SET_KEYBIT, atoi(input_preset_value_get("touch_arrows_left_key")));
			if(input_preset_value_isset("touch_arrows_right_key"))
				ioctl(input, UI_SET_KEYBIT, atoi(input_preset_value_get("touch_arrows_right_key")));
		}
		if(input_preset_value_comp("touch_action", "key"))
		{
			if(input_preset_value_isset("touch_key"))
				ioctl(input, UI_SET_KEYBIT, atoi(input_preset_value_get("touch_key")));
		}
	}
	if(input_preset_value_bool("use_button_aux"))
	{
		if(input_preset_value_comp("button_aux_action", "key") && input_preset_value_isset("button_aux_key"))
		{
			ioctl(input, UI_SET_KEYBIT, atoi(input_preset_value_get("button_aux_key")));
		}
	}
	if(input_preset_value_bool("use_button_power"))
	{
		if(input_preset_value_comp("button_power_action", "key") && input_preset_value_isset("button_power_key"))
		{
			ioctl(input, UI_SET_KEYBIT, atoi(input_preset_value_get("button_power_key")));
		}
	}

	if(input_device == NULL)
	{
		memset(&uinput_device, 0, sizeof(uinput_device));
		snprintf(uinput_device.name, UINPUT_MAX_NAME_SIZE, "uinput-sample");
		uinput_device.id.bustype = BUS_USB;
		uinput_device.id.vendor  = 0x1;
		uinput_device.id.product = 0x1;
		uinput_device.id.version = 1;

		write(input, &uinput_device, sizeof(uinput_device));

		ioctl(input, UI_DEV_CREATE);
	}

	/*** END : Testing purpose only ***/
}

void input_event_get(void)
{
	if(!sock_fd)
	{
		printf("Socket error.");
		exit(1);
	}

	struct input_event_light event_light;
	struct input_event event;

	__s32 event_light_value_old_0=0;
	__s32 event_light_value_old_1=0;

	/* X and Y start values for touch_arrows preset */
	__s32 touch_arrows_x_svalue=0;
	__s32 touch_arrows_y_svalue=0;

	/* X and Y last values for touch_arrows preset */
	__s32 touch_arrows_x_lvalue=0;
	__s32 touch_arrows_y_lvalue=0;

	/* How many times does the user stoped ? */
	int touch_arrows_x_stop=0;
	int touch_arrows_y_stop=0;

	int touch_arrows_x_lock=false;
	int touch_arrows_y_lock=false;



	int up_lock=0;
	int down_lock=0;
	int right_lock=0;
	int left_lock=0;
	int shift_lock=0;
	int pause_lock=1;

	while(read(sock_fd, &event_light, sizeof(struct input_event_light)))
	{
		event.type=event_light.type;
		event.code=event_light.code;
		event.value=event_light.value;

	//	printf("--> %d %d %d\n", event_light.type, event_light.code, event_light.value);

		if(event_light.type == 3)
		{
			if(input_preset_value_bool("use_touch"))
			{
				if(input_preset_value_comp("touch_action", "mouse"))
				{
					event.type=EV_REL;
		
					if(event_light.code == 0)
					{
						if(!event_light_value_old_0)
							event_light_value_old_0=event_light.value;

						event.code=REL_Y;
						event.value=event_light_value_old_0 - event_light.value;
						write(input, &event, sizeof(struct input_event));

						event_light_value_old_0=event_light.value;
					}
					else if(event_light.code == 1)
					{
						if(!event_light_value_old_1)
							event_light_value_old_1=event_light.value;

						event.code=REL_X;
						event.value=(event_light_value_old_1 - event_light.value) * -1;
						write(input, &event, sizeof(struct input_event));

						event_light_value_old_1=event_light.value;
					}
					else if(event_light.code == 24 && event_light.value == 0)
					{
						event_light_value_old_0=0;
						event_light_value_old_1=0;
					}
				}
				if(input_preset_value_comp("touch_action", "mouse_rotated"))
				{
					event.type=EV_REL;
		
					if(event_light.code == 0)
					{
						if(!event_light_value_old_0)
							event_light_value_old_0=event_light.value;

						event.code=REL_X;
						event.value=event_light_value_old_0 - event_light.value;
						write(input, &event, sizeof(struct input_event));

						event_light_value_old_0=event_light.value;
					}
					else if(event_light.code == 1)
					{
						if(!event_light_value_old_1)
							event_light_value_old_1=event_light.value;

						event.code=REL_Y;
						event.value=event_light_value_old_1 - event_light.value;
						write(input, &event, sizeof(struct input_event));

						event_light_value_old_1=event_light.value;
					}
					else if(event_light.code == 24 && event_light.value == 0)
					{
						event_light_value_old_0=0;
						event_light_value_old_1=0;
					}
				}
				if(input_preset_value_comp("touch_action", "key"))
				{
					if(!input_preset_value_isset("touch_key"))
					{
						printf("Missing preset value: 'touch_key'\n");
						continue;
					}

					event.type=EV_KEY;
					event.code=atoi(input_preset_value_get("touch_key"));

					if(event_light.code == 24 && event_light.value == 1)
					{
						event.value = 1;
						write(input, &event, sizeof(struct input_event));
					}
					else if(event_light.code == 24 && event_light.value == 0)
					{
						event.value = 0;
						write(input, &event, sizeof(struct input_event));
					}
				}
				if(input_preset_value_comp("touch_action", "arrows"))
				{
					event.type=EV_REL;
					event.type=EV_KEY;

					if(input_preset_value_bool("touch_arrows_hold"))
					{
						if(event_light.code == 0)
						{
							if(!touch_arrows_x_svalue)
							{
								touch_arrows_x_svalue=event_light.value;
								continue;
							}

							if(!touch_arrows_x_lock && !touch_arrows_y_lock && touch_arrows_x_stop > 8 && (abs(touch_arrows_x_svalue - touch_arrows_x_lvalue) > abs(touch_arrows_y_svalue - touch_arrows_y_lvalue)))
							{
								if(input_preset_value_isset("touch_arrows_up_key") && (touch_arrows_x_svalue - touch_arrows_x_lvalue) < 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_up_key"));
									event.value = 1;
									write(input, &event, sizeof(struct input_event));
								}
								else if(input_preset_value_isset("touch_arrows_down_key") && (touch_arrows_x_svalue - touch_arrows_x_lvalue) > 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_down_key"));
									event.value = 1;
									write(input, &event, sizeof(struct input_event));
								}

								touch_arrows_x_lock=true;
								continue;
							}

							if(abs(touch_arrows_x_lvalue - event_light.value) < 10)
								touch_arrows_x_stop++;
							else
								if(touch_arrows_x_lock || touch_arrows_y_lock)
								{
									if(touch_arrows_x_lock)
									{
										if((touch_arrows_x_svalue - touch_arrows_x_lvalue) < 0)
										{
											event.code=atoi(input_preset_value_get("touch_arrows_up_key"));
											event.value = 0;
											write(input, &event, sizeof(struct input_event));
										}
										else
										{
											event.code=atoi(input_preset_value_get("touch_arrows_down_key"));
											event.value = 0;
											write(input, &event, sizeof(struct input_event));
										}
									}
									if(touch_arrows_y_lock)
									{
										if(input_preset_value_isset("touch_arrows_right_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) < 0)
										{
											event.code=atoi(input_preset_value_get("touch_arrows_right_key"));
											event.value = 0;
											write(input, &event, sizeof(struct input_event));
										}
										else if(input_preset_value_isset("touch_arrows_left_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) > 0)
										{
											event.code=atoi(input_preset_value_get("touch_arrows_left_key"));
											event.value = 0;
											write(input, &event, sizeof(struct input_event));
										}
									}

									touch_arrows_x_stop=0;
									touch_arrows_x_lock=false;

									touch_arrows_y_stop=0;
									touch_arrows_y_lock=false;
	
									touch_arrows_x_svalue=0;
									touch_arrows_y_svalue=0;
								}

							touch_arrows_x_lvalue=event_light.value;
						}
						else if(event_light.code == 1)
						{
							if(!touch_arrows_y_svalue)
							{
								touch_arrows_y_svalue=event_light.value;
								continue;
							}

							if(!touch_arrows_x_lock && !touch_arrows_y_lock && touch_arrows_y_stop > 8 && (abs(touch_arrows_x_svalue - touch_arrows_x_lvalue) < abs(touch_arrows_y_svalue - touch_arrows_y_lvalue)))
							{
								if(input_preset_value_isset("touch_arrows_right_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) < 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_right_key"));
									event.value = 1;
									write(input, &event, sizeof(struct input_event));
								}
								else if(input_preset_value_isset("touch_arrows_left_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) > 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_left_key"));
									event.value = 1;
									write(input, &event, sizeof(struct input_event));
								}

								touch_arrows_y_lock=true;
								continue;
							}

							if(abs(touch_arrows_y_lvalue - event_light.value) < 10)
								touch_arrows_y_stop++;
							else
								if(touch_arrows_x_lock || touch_arrows_y_lock)
								{
									if(touch_arrows_x_lock)
									{
										if((touch_arrows_x_svalue - touch_arrows_x_lvalue) < 0)
										{
											event.code=atoi(input_preset_value_get("touch_arrows_up_key"));
											event.value = 0;
											write(input, &event, sizeof(struct input_event));
										}
										else
										{
											event.code=atoi(input_preset_value_get("touch_arrows_down_key"));
											event.value = 0;
											write(input, &event, sizeof(struct input_event));
										}
									}
									if(touch_arrows_y_lock)
									{
										if(input_preset_value_isset("touch_arrows_right_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) < 0)
										{
											event.code=atoi(input_preset_value_get("touch_arrows_right_key"));
											event.value = 0;
											write(input, &event, sizeof(struct input_event));
										}
										else if(input_preset_value_isset("touch_arrows_left_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) > 0)
										{
											event.code=atoi(input_preset_value_get("touch_arrows_left_key"));
											event.value = 0;
											write(input, &event, sizeof(struct input_event));
										}
									}

									touch_arrows_x_stop=0;
									touch_arrows_x_lock=false;

									touch_arrows_y_stop=0;
									touch_arrows_y_lock=false;
	
									touch_arrows_x_svalue=0;
									touch_arrows_y_svalue=0;
								}

							touch_arrows_y_lvalue=event_light.value;
						}
						else if(event_light.code == 24 && event_light.value == 0)
						{
							if(touch_arrows_x_lock)
							{
								if((touch_arrows_x_svalue - touch_arrows_x_lvalue) < 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_up_key"));
									event.value = 0;
									write(input, &event, sizeof(struct input_event));
								}
								else
								{
									event.code=atoi(input_preset_value_get("touch_arrows_down_key"));
									event.value = 0;
									write(input, &event, sizeof(struct input_event));
								}
							}
							if(touch_arrows_y_lock)
							{
								if(input_preset_value_isset("touch_arrows_right_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) < 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_right_key"));
									event.value = 0;
									write(input, &event, sizeof(struct input_event));
								}
								else if(input_preset_value_isset("touch_arrows_left_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) > 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_left_key"));
									event.value = 0;
									write(input, &event, sizeof(struct input_event));
								}
							}

							touch_arrows_x_stop=0;
							touch_arrows_x_lock=false;

							touch_arrows_y_stop=0;
							touch_arrows_y_lock=false;
	
							touch_arrows_x_svalue=0;
							touch_arrows_y_svalue=0;
						}
					}
					else /* input_arrows_hold = false */
					{
						if(event_light.code == 0)
						{
							if(!touch_arrows_x_svalue)
								touch_arrows_x_svalue=event_light.value;
							else
								touch_arrows_x_lvalue=event_light.value;

						}
						else if(event_light.code == 1)
						{
							if(!touch_arrows_y_svalue)
								touch_arrows_y_svalue=event_light.value;
							else
								touch_arrows_y_lvalue=event_light.value;
						}
						else if(event_light.code == 24 && event_light.value == 0)
						{
							if(abs(touch_arrows_x_svalue - touch_arrows_x_lvalue) > abs(touch_arrows_y_svalue - touch_arrows_y_lvalue))
							{
								if(input_preset_value_isset("touch_arrows_up_key") && (touch_arrows_x_svalue - touch_arrows_x_lvalue) < 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_up_key"));

									event.value = 1;
									write(input, &event, sizeof(struct input_event));

									event.value = 0;
									write(input, &event, sizeof(struct input_event));
								}
								else if(input_preset_value_isset("touch_arrows_down_key") && (touch_arrows_x_svalue - touch_arrows_x_lvalue) > 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_down_key"));

									event.value = 1;
									write(input, &event, sizeof(struct input_event));

									event.value = 0;
									write(input, &event, sizeof(struct input_event));
								}
							}
							else
							{
								if(input_preset_value_isset("touch_arrows_right_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) < 0)
								{
									event.code=atoi(input_preset_value_get("touch_arrows_right_key"));

									event.value = 1;
									write(input, &event, sizeof(struct input_event));

									event.value = 0;
									write(input, &event, sizeof(struct input_event));
								}
								else if(input_preset_value_isset("touch_arrows_left_key") && (touch_arrows_y_svalue - touch_arrows_y_lvalue) > 0)
								{
										event.code=atoi(input_preset_value_get("touch_arrows_left_key"));

									event.value = 1;
									write(input, &event, sizeof(struct input_event));

									event.value = 0;
									write(input, &event, sizeof(struct input_event));
								}
							}
		
							touch_arrows_x_svalue=0;
							touch_arrows_y_svalue=0;
						}
					}
				}
			}
		}

		if(event_light.type == 1 && event_light.code != 330)
		{
			if(event_light.code == 169)
			{
				if(input_preset_value_bool("use_button_aux"))
				{
					if(input_preset_value_comp("button_aux_action", "key"))
					{
						if(!input_preset_value_isset("button_aux_key"))
						{
							printf("Missing preset value: 'button_aux_key'\n");
							continue;
						}

						event.type=EV_KEY;
						event.code=atoi(input_preset_value_get("button_aux_key"));

						if(event_light.value == 1)
						{
							event.value = 1;
							write(input, &event, sizeof(struct input_event));
						}
						else if(event_light.value == 0)
						{
							event.value = 0;
							write(input, &event, sizeof(struct input_event));
						}
					}
				}
			}
			if(event_light.code == 116)
			{
				if(input_preset_value_bool("use_button_power"))
				{
					if(input_preset_value_comp("button_power_action", "key"))
					{
						if(!input_preset_value_isset("button_power_key"))
						{
							printf("Missing preset value: 'button_power_key'\n");
							continue;
						}

						event.type=EV_KEY;
						event.code=atoi(input_preset_value_get("button_power_key"));

						if(event_light.value == 1)
						{
							event.value = 1;
							write(input, &event, sizeof(struct input_event));
						}
						else if(event_light.value == 0)
						{
							event.value = 0;
							write(input, &event, sizeof(struct input_event));
						}
					}
				}
			}
		}

		if(event_light.type == EV_SYN && event_light.code == SYN_REPORT && event_light.value == 0)
		{
			event.type = EV_SYN;
			event.code = SYN_REPORT;
			event.value = 0;
			write(input, &event, sizeof(struct input_event));
		}
	}
}

void input_write_close(void)
{
	if(input_device == NULL)
		ioctl(input, UI_DEV_DESTROY);
	close(input);
}
